@@ -1,5 +1,7 @@
 #! /usr/bin/perl -w
 
+# $Id: copy_user.pl,v 1.3 2000/11/11 07:48:59 rvsutherland Exp $
+
 use strict;
 
 use DBI;
@@ -33,7 +35,7 @@ DDL::Oracle->configure(
 # allows the user to see the prompts.
 
 print STDERR "\nEnter Name of Existing User (the Template) : ";
-chop( $old_user = lc( <STDIN> ) );
+chomp( $old_user = lc( <STDIN> ) );
 die "\nYou must specify an Existing User.\n" unless $old_user;
 print STDERR "\n";
 
@@ -52,7 +54,7 @@ $old_sql = $obj->create;   # Will FAIL unless $old_user exists!
 while (1)
 {
   print STDERR "Enter Name of New User or <ENTER> when done: ";
-  chop( $new_user = lc( <STDIN> ) );
+  chomp( $new_user = lc( <STDIN> ) );
   last unless $new_user;
   push @users, $new_user;
 }
@@ -81,3 +83,11 @@ foreach $new_user( @users )
 
 print $ddl;
 
+# $Log: copy_user.pl,v $
+# Revision 1.3  2000/11/11 07:48:59  rvsutherland
+# Added CVS tags
+#
+# Revision 1.2  2000/11/11 00:20:42  rvsutherland
+# Initial revision
+#
+
